import { Injector, InjectionToken } from '@angular/core';
import { Router } from '@angular/router';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { StateService } from '../common/state-service.abstract';
import { Action } from './action.model';
import { ExtensionPoint, ExtensionFactory } from '../common/extension-hooks';
/**
 * An extension HOOK can use either a pure value:
 * ```typescript
 *  { provide: HOOK_X, useValue: { ...hookValue }, multi: true }
 * ```
 *
 * Or an array to directly register multiple:
 * ```typescript
 *  { provide: HOOK_X, useValue: [{ ...hookValues }], multi: true }
 * ```
 *
 * Or an ExtensionFactory which allows to define a get() function. This function
 * gets called on each navigation with the current route and can return values
 * async (observable or promise).
 * ```typescript
 *  { provide: HOOK_X, useFactory: { get: (route) => doSomethingAsync(route) }, multi: true }
 * ```
 */
export declare type ActionExtension = Action | Action[] | ExtensionFactory<Action>;
/**
 * A hook to add an action using the multi-provider extension concept.
 */
export declare const HOOK_ACTION: InjectionToken<unknown>;
/**
 * A service which defines global actions.
 */
export declare class ActionService extends StateService implements ExtensionPoint<ActionExtension> {
    private injector;
    items$: Observable<Action[]>;
    factories: ActionExtension[];
    readonly refreshTrigger: Subject<unknown>;
    readonly state$: BehaviorSubject<Set<Action>>;
    constructor(injector: Injector, router: Router);
    /**
     * Refresh the extension factories
     */
    refresh(): void;
    /**
     * Returns the current state.
     * @readonly
     * @returns The current set of actions.
     */
    get state(): Set<Action>;
    /**
     * Adds a new item to the actions in the header and emits a state change.
     * @param item The item to add.
     */
    add(item: Action): void;
    /**
     * Removes an action from the header and emits a state change.
     * @param item The item to remove.
     */
    remove(item: Action): void;
}
