import { InjectionToken, Injector } from '@angular/core';
import { Router } from '@angular/router';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { ExtensionFactory, ExtensionPoint } from '../common/extension-hooks';
import { StateService } from '../common/state-service.abstract';
import { NavigatorNode } from './navigator-node';
/**
 * An extension HOOK can use either a pure value:
 * ```typescript
 *  { provide: HOOK_X, useValue: { ...hookValue }, multi: true }
 * ```
 *
 * Or an array to directly register multiple:
 * ```typescript
 *  { provide: HOOK_X, useValue: [{ ...hookValues }], multi: true }
 * ```
 *
 * Or an ExtensionFactory which allows to define a get() function. This function
 * gets called on each navigation with the current route and can return values
 * async (observable or promise).
 * ```typescript
 *  { provide: HOOK_X, useFactory: { get: (route) => doSomethingAsync(route) }, multi: true }
 * ```
 */
declare type NavigatorExtension = NavigatorNode | NavigatorNode[] | ExtensionFactory<NavigatorNode>;
/**
 * A hook to use for Multi Provider extension.
 */
export declare const HOOK_NAVIGATOR_NODES: InjectionToken<NavigatorExtension[]>;
/**
 * A service which defines the navigator.
 */
export declare class NavigatorService extends StateService implements ExtensionPoint<NavigatorExtension> {
    private injector;
    private router;
    /**
     * Navigator menu items observable. It emits the current list of nodes in the navigator.
     */
    items$: Observable<NavigatorNode[]>;
    /**
     * Refresh the extension factories subject.
     * @readonly
     */
    readonly refreshTrigger: Subject<unknown>;
    /**
     * @ignore
     */
    readonly state$: BehaviorSubject<Set<NavigatorNode>>;
    /**
     * Indicates whether the menu entry associated with the given URL should be expanded.
     */
    firstUrl: boolean;
    /**
     * Additional factories that can be added by plugins.
     */
    factories: NavigatorExtension[];
    constructor(injector: Injector, router: Router);
    /**
     * Checks whenever a navigator entry associated with the given URL should be expanded.
     * @param {NavigatorNode} node Navigator node.
     */
    openOnFirstUrl(node: NavigatorNode): boolean;
    /**
     * Returns the current state.
     * @readonly
     * @returns The current set of actions.
     */
    get state(): Set<NavigatorNode>;
    /**
     * Refresh the extension factories.
     */
    refresh(): void;
    /**
     * Adds a new node to the navigator.
     * @param {NavigatorNode} node Navigator node to add.
     */
    add(node: NavigatorNode): void;
    /**
     * Removes a node from the navigator.
     * @param {NavigatorNode} node Navigator node to remove.
     */
    remove(node: NavigatorNode): void;
}
export {};
