import { InjectionToken, Injector } from '@angular/core';
import { Router } from '@angular/router';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { ExtensionFactory, ExtensionPoint } from '../common/extension-hooks';
import { StateService } from '../common/state-service.abstract';
import { Search } from './search.model';
/**
 * An extension HOOK can use either a pure value:
 * ```typescript
 *  { provide: HOOK_X, useValue: { ...hookValue }, multi: true }
 * ```
 *
 * Or an array to directly register multiple:
 * ```typescript
 *  { provide: HOOK_X, useValue: [{ ...hookValues }], multi: true }
 * ```
 *
 * Or an ExtensionFactory which allows to define a get() function. This function
 * gets called on each navigation with the current route and can return values
 * async (observable or promise).
 * ```typescript
 *  { provide: HOOK_X, useFactory: { get: (route) => doSomethingAsync(route) }, multi: true }
 * ```
 */
declare type SearchExtension = Search | Search[] | ExtensionFactory<Search>;
/**
 * A hook to use for Multi Provider extension.
 */
export declare const HOOK_SEARCH: InjectionToken<unknown>;
/**
 * A service which defines the search function.
 */
export declare class SearchService extends StateService implements ExtensionPoint<SearchExtension> {
    private injector;
    items$: Observable<Search[]>;
    /**
     * Additional factories that can be added by plugins.
     */
    factories: SearchExtension[];
    readonly refreshTrigger: Subject<unknown>;
    protected state$: BehaviorSubject<Set<Search>>;
    constructor(injector: Injector, router: Router);
    /**
     * Refresh the extension factories
     */
    refresh(): void;
    /**
     * Returns the current state.
     * @readonly
     * @returns The current set of actions.
     */
    get state(): Set<Search>;
    /**
     * Adds a new item to the search in the header.
     * @param item The item to add.
     */
    add(item: Search): void;
    /**
     * Removes an search from the header.
     * @param item The item to remove.
     */
    remove(item: Search): void;
}
export {};
