import { EventEmitter } from '@angular/core';
import { TranslateService } from '@ngx-translate/core';
import { ISelectModalObject, ISelectModalOption, ModalLabels, ModalSelectionMode } from '@c8y/ngx-components';
import { IManagedObject } from '@c8y/client';
import { Subject, Observable, BehaviorSubject } from 'rxjs';
import { RepositoryService } from '../repository.service';
import { RepositoryType, RepositoryCategory, RepositoryBinary, SelectedRepositoryBinary } from '../repository.model';
/**
 * RepositorySelectModalComponent displays repository entries options and allows to select them.
 *
 * @example
 * ```
 * import { take } from 'rxjs/operators';
 * import { RepositorySelectModalComponent, ModalSelectionMode, RepositoryType } from '@c8y/ngx-components/repository';
 *
 * const initialState = {
 *   repositoryType: RepositoryType.FIRMWARE,
 *   title: gettext('Install firmware'),
 *   subTitle: gettext('Available firmwares matching the device type'),
 *   icon: 'c8y-firmware',
 *   mode: ModalSelectionMode.SINGLE,
 *   labels: { ok: gettext('Install') },
 *   disableSelected: false
 * };
 *
 * const modal = this.bsModal.show(RepositorySelectModalComponent, {
 *   ignoreBackdropClick: true,
 *   initialState
 * });
 *
 * modal.content.load.next();
 * modal.content.resultEmitter.pipe(take(1)).subscribe((firmware) => {
 *   ...
 * })
 * ```
 */
export declare class RepositorySelectModalComponent {
    private repositoryService;
    private translateService;
    /**
     * Optional
     * Allows to provide custom data.
     * @example
     * ```
     * import { from } from 'rxjs';
     *
     * const repositoryEntry = { name: 'ExampleEntry', type: 'c8y_Firmware' };
     * const versions = [{ c8y_Firmware: { version: '1.0.0', url: 'http://example.com' } }];
     *
     * const initialState = {repositoryEntriesWithVersions$: from({ ...repositoryEntry, versions })};
     * ```
     */
    repositoryEntriesWithVersions$: Observable<IManagedObject[]>;
    /**
     * Repository entry type.
     */
    repositoryType: RepositoryType.FIRMWARE | RepositoryType.SOFTWARE;
    /**
     * Optional
     * Allows to use custom badges templates.
     * @example
     * ```
     * import { gettext } from '@c8y/ngx-components';
     *
     * const badgeTemplates = { '=1': gettext('{{count}} version'), other: gettext('{{count}} versions') };
     * const initialState = { badgeTemplates };
     * ```
     */
    badgeTemplates: {
        '=1': any;
        other: any;
    };
    /**
     * Optional
     * Allows to provide custom modal title.
     */
    title: string;
    /**
     * Optional
     * Allows to provide custom modal subtitle.
     */
    subTitle: string;
    /**
     * Loads the content of the modal.
     * Must be invoked by the modal's caller.
     */
    load: Subject<void>;
    /**
     * Optional
     * Emits a search string currently entered in the filter input.
     * Use it to filter the items if you use custom repositoryEntriesWithVersions$.
     */
    searchTerm: BehaviorSubject<string>;
    /**
     * Optional
     * Allows to provide device type query to restrict search criteria.
     * Only takes effect when repositoryEntriesWithVersions$ is not provided,
     * otherwise modal's caller have to provide already filtered data in the repositoryEntriesWithVersions$.
     */
    deviceTypeQuery: any;
    /**
     * Optional
     * Allows to provide query to restrict search criteria.
     * Only takes effect when repositoryEntriesWithVersions$ is not provided,
     * otherwise modal's caller have to provide already filtered data in the repositoryEntriesWithVersions$.
     */
    searchQuery: any;
    /**
     * Optional
     * Allows to provide custom labels for the buttons responsible for confirm/dismiss modal actions.
     */
    labels: ModalLabels;
    /**
     * Optional
     * Allows to hide the filter input field.
     * By default, the filter input field is displayed.
     */
    showFilter: boolean;
    /**
     * Optional
     * Allows to show a warning that the search criteria should be narrowed down.
     * By default, this warning is hidden.
     */
    areMoreEntries: boolean;
    /**
     * Optional
     * Allows to pass the array of items. Each item from this array will be marked as selected in the modal.
     */
    selected: SelectedRepositoryBinary[];
    /**
     * Emits the list of selected options.
     */
    resultEmitter: EventEmitter<SelectedRepositoryBinary[]>;
    /**
     * Optional
     * Allows to change selection mode.
     * Supported options:
     *   * single: only single option can be selected.
     *   * multiple: multiple options can be selected.
     */
    mode: ModalSelectionMode;
    /**
     * Optional
     * Allows to use custom icon in the modal header.
     */
    icon: string;
    /**
     * Allows to block selection of the other versions from the same repository entry.
     */
    disableSelected: boolean;
    modalEntries: Observable<ISelectModalObject[]>;
    private PAGE_SIZE;
    private queriesUtil;
    constructor(repositoryService: RepositoryService, translateService: TranslateService);
    ngOnInit(): void;
    getAndAssignRepositoryBinaries(mos: IManagedObject[]): IManagedObject[];
    search(searchTerm: string): void;
    result(selectedItems: SelectedRepositoryBinary[]): void;
    aggregate(mos: IManagedObject[]): Promise<ISelectModalObject[]>;
    getSelectModalOptions(versions: RepositoryBinary[], selectedItems: SelectedRepositoryBinary[], repositoryEntry: RepositoryCategory, repositoryType: RepositoryType): ISelectModalOption[];
    isBinaryRepositorySelected(selectedItems: SelectedRepositoryBinary[], repositoryEntry: RepositoryCategory, repositoryBinary: RepositoryBinary, repositoryType: RepositoryType): boolean;
    getSelectModalObject(repositoryEntry: RepositoryCategory, options: ISelectModalOption[]): ISelectModalObject;
}
